@@ -1,6 +1,6 @@
 
 create table five (five_col int);
 
-comment on table five is '2+3=5';
+comment on table five is '5 ± 1 = 4 or 6';
 comment on column five.five_col is 'this is a column for fives';
 
